package premium_store.controller.controller;

import org.springframework.web.bind.annotation.*;
import premium_store.controller.DTO.ClientDTO;
import premium_store.model.Tank;
import premium_store.service.GameClientService;

import java.util.List;

@RestController
@RequestMapping("/client")
public class GameClientController {
    private final GameClientService gameClientService;

    public GameClientController(GameClientService gameClientService){
        this.gameClientService = gameClientService;
    }

    //аннотация PathVariable связывает значения id из URL и Long id
    @GetMapping("/{id}")
    public ClientDTO getClient(@PathVariable Long id) {
        return new ClientDTO(gameClientService.findClient(id));
    }

    //с помощью Java Stream преобразуем набор пришедших данных в объекты StudentDto
    @GetMapping("/")
    public List<ClientDTO> getClients() {
        return gameClientService.findAllClients().stream()
                .map(ClientDTO::new)
                .toList();
    }

    @PostMapping("/")
    public ClientDTO createClient(@RequestParam("nickName") String nickName,
                                  @RequestParam("email") String email,
                                  @RequestParam("balance") Integer balance) {
        return new ClientDTO(gameClientService.addClient(nickName, email, balance));
    }

    @PutMapping("/{id}")
    public ClientDTO updateClient(@PathVariable Long id,
                                  @RequestParam("nickName") String nickName,
                                  @RequestParam("email") String email,
                                  @RequestParam("balance") Integer balance,
                                  @RequestParam("tanks") List<Tank> tanks) {
        return new ClientDTO(gameClientService.updateClient(id, nickName, email, balance, tanks));
    }

    @DeleteMapping("/{id}")
    public ClientDTO deleteClient(@PathVariable Long id) {
        return new ClientDTO(gameClientService.deleteClient(id));
    }
}
