package premium_store.controller.controller;

import org.springframework.web.bind.annotation.*;
import premium_store.model.Nation;
import premium_store.service.NationService;

import java.util.List;

//привязываем наш контроллер к придуманному корневому URL благодаря аннотациям
//здесь происходит внедрение зависимости нашего сервиса
//так же здесь прописываем вызовы методов CRUD в привязке к URL
@RestController
@RequestMapping("/nation")
public class NationController {
    private final NationService nationService;

    public NationController(NationService nationService){
        this.nationService = nationService;
    }

    //аннотация PathVariable связывает значения id из URL и Long id
    @GetMapping("/{id}")
    public Nation getNation(@PathVariable Long id) {
        return nationService.findNation(id);
    }

    //с помощью Java Stream преобразуем набор пришедших данных в объекты StudentDto
    @GetMapping("/")
    public List<Nation> getNations() {
        return nationService.findAllNations().stream()
                .toList();
    }

    @PostMapping("/")
    public Nation createNation(@RequestParam("nation") String nation) {
        return nationService.addNation(nation);
    }

    @PutMapping("/{id}")
    public Nation updateNation(@PathVariable Long id,
                             @RequestParam("nation") String nation) {
        return nationService.updateNation(id, nation);
    }

    @DeleteMapping("/{id}")
    public Nation deleteNation(@PathVariable Long id) {
        return nationService.deleteNation(id);
    }
}

