package premium_store.controller.controller;

import org.springframework.web.bind.annotation.*;
import premium_store.controller.DTO.TankDTO;
import premium_store.model.GameClient;
import premium_store.model.TankLevel;
import premium_store.model.Nation;
import premium_store.service.TankService;

import java.util.List;

@RestController
@RequestMapping("/tank")
public class TankController {
    private final TankService tankService;

    public TankController(TankService tankService){
        this.tankService = tankService;
    }

    //аннотация PathVariable связывает значения id из URL и Long id
    @GetMapping("/{id}")
    public TankDTO getTank(@PathVariable Long id) {
        return new TankDTO(tankService.findTank(id));
    }

    //с помощью Java Stream преобразуем набор пришедших данных в объекты TankDTO
    @GetMapping("/")
    public List<TankDTO> getTanks() {
        return tankService.findAllTanks().stream()
                .map(TankDTO::new)
                .toList();
    }

    @PostMapping("/")
    public TankDTO createTank(@RequestParam("firstName") String name,
                              @RequestParam("nation") Nation nation,
                              @RequestParam("level") TankLevel tankLevel,
                              @RequestParam("cost") int cost) {
        return new TankDTO(tankService.addTank(name, nation, tankLevel, cost));
    }

    @PutMapping("/{id}")
    public TankDTO updateTank(@PathVariable Long id,
                              @RequestParam("firstName") String name,
                              @RequestParam("nation") Nation nation,
                              @RequestParam("level") TankLevel tankLevel,
                              @RequestParam("cost") int cost,
                              @RequestParam("clients")List<GameClient> gameClients) {
        return new TankDTO(tankService.updateTank(id, name, nation, tankLevel, cost, gameClients));
    }

    @DeleteMapping("/{id}")
    public TankDTO deleteTank(@PathVariable Long id) {
        return new TankDTO(tankService.deleteTank(id));
    }
}
