package premium_store.controller.controller;

import org.springframework.web.bind.annotation.*;
import premium_store.model.TankLevel;
import premium_store.service.TankLevelService;

import java.util.List;

//привязываем наш контроллер к придуманному корневому URL благодаря аннотациям
//здесь происходит внедрение зависимости нашего сервиса
//так же здесь прописываем вызовы методов CRUD в привязке к URL
@RestController
@RequestMapping("/level")
public class TankLevelController {
    private final TankLevelService tankLevelService;

    public TankLevelController(TankLevelService tankLevelService){
        this.tankLevelService = tankLevelService;
    }

    //аннотация PathVariable связывает значения id из URL и Long id
    @GetMapping("/{id}")
    public TankLevel getLevel(@PathVariable Long id) {
        return tankLevelService.findLevel(id);
    }

    //с помощью Java Stream преобразуем набор пришедших данных в объекты StudentDto
    @GetMapping("/")
    public List<TankLevel> getLevels() {
        return tankLevelService.findAllLevels().stream()
                .toList();
    }

    @PostMapping("/")
    public TankLevel createLevel(@RequestParam("Level") int level) {
        return tankLevelService.addLevel(level);
    }

    @PutMapping("/{id}")
    public TankLevel updateLevel(@PathVariable Long id,
                                 @RequestParam("Level") int level) {
        return tankLevelService.updateLevel(id, level);
    }

    @DeleteMapping("/{id}")
    public TankLevel deleteLevel(@PathVariable Long id) {
        return tankLevelService.deleteLevel(id);
    }
}
