package premium_store.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import premium_store.model.GameClient;
import premium_store.model.Tank;
import premium_store.repository.GameClientRepository;
import premium_store.service.exception.ClientNotFoundException;
import premium_store.util.validation.ValidatorUtil;

import java.util.List;
import java.util.Optional;

@Service
public class GameClientService {
    private final GameClientRepository gameClientRepository;
    private final ValidatorUtil validatorUtil;

    public GameClientService(GameClientRepository gameClientRepository, ValidatorUtil validatorUtil){
        this.gameClientRepository = gameClientRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public GameClient addClient(String newNickName, String newEmail, Integer newBallance) {
        final GameClient gameClient = new GameClient(newNickName, newEmail, newBallance);
        validatorUtil.validate(gameClient);

        return gameClientRepository.save(gameClient);
    }

    @Transactional(readOnly = true)
    public GameClient findClient(Long id) {
        final Optional<GameClient> client = gameClientRepository.findById(id);

        return client.orElseThrow(() -> new ClientNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<GameClient> findAllClients() {
        return gameClientRepository.findAll();
    }

    @Transactional
    public GameClient updateClient(Long id, String newNickName, String newEmail, Integer newBalance, List<Tank> newTanks) {
        final GameClient currentGameClient = findClient(id);
        currentGameClient.setNickName(newNickName);
        currentGameClient.setEmail(newEmail);
        currentGameClient.setBalance(newBalance);
        currentGameClient.setTanks(newTanks);
        validatorUtil.validate(currentGameClient);

        return gameClientRepository.save(currentGameClient);
    }

    @Transactional
    public GameClient deleteClient(Long id) {
        final GameClient currentGameClient = findClient(id);
        gameClientRepository.delete(currentGameClient);

        return currentGameClient;
    }

    //прямой sql-запрос на удаление всех записей в таблице
    @Transactional
    public void deleteAllClients() {
        gameClientRepository.deleteAll();
    }
}
