package premium_store.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import premium_store.model.GameClient;
import premium_store.model.TankLevel;
import premium_store.model.Nation;
import premium_store.model.Tank;
import premium_store.repository.TankRepository;
import premium_store.service.exception.TankNotFoundException;
import premium_store.util.validation.ValidatorUtil;

import java.util.List;
import java.util.Optional;

@Service
public class TankService {
    private final TankRepository tankRepository;
    private final ValidatorUtil validatorUtil;

    public TankService(TankRepository tankRepository, ValidatorUtil validatorUtil){
        this.tankRepository = tankRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Tank addTank(String newName, Nation newNation, TankLevel newTankLevel, int newCost) {
        final Tank tank = new Tank(newName, newNation, newTankLevel, newCost);
        validatorUtil.validate(tank);

        return tankRepository.save(tank);
    }

    @Transactional(readOnly = true)
    public Tank findTank(Long id) {
        final Optional<Tank> tank = tankRepository.findById(id);

        return tank.orElseThrow(() -> new TankNotFoundException(id));
    }

    @Transactional(readOnly = true)
    public List<Tank> findAllTanks() {
        return tankRepository.findAll();
    }

    @Transactional
    public Tank updateTank(Long id, String newName, Nation newNation,
                           TankLevel newTankLevel, int newCost, List<GameClient> newGameClients) {
        final Tank currentTank = findTank(id);
        currentTank.setName(newName);
        currentTank.setNation(newNation);
        currentTank.setLevel(newTankLevel);
        currentTank.setCost(newCost);
        currentTank.setClients(newGameClients);
        validatorUtil.validate(currentTank);

        return tankRepository.save(currentTank);
    }

    @Transactional
    public Tank deleteTank(Long id) {
        final Tank currentTank = findTank(id);
        tankRepository.delete(currentTank);

        return currentTank;
    }

    //запрос на удаление всех записей в таблице
    @Transactional
    public void deleteAllTanks() {
        tankRepository.deleteAll();
    }
}
