package premium_store;

import premium_store.model.GameClient;
import premium_store.model.TankLevel;
import premium_store.model.Nation;
import premium_store.model.Tank;
import premium_store.service.GameClientService;
import premium_store.service.TankLevelService;
import premium_store.service.NationService;
import premium_store.service.TankService;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import premium_store.service.exception.ClientNotFoundException;

import java.util.*;

@SpringBootTest
class PremiumStoreApplicationTests {
    private static final Logger log = LoggerFactory.getLogger(PremiumStoreApplicationTests.class);

	@Autowired
    TankService tankService;

    @Autowired
	GameClientService gameClientService;

	@Autowired
	TankLevelService tankLevelService;

	@Autowired
    NationService nationService;

    @Test
    void testClientRead() {
        gameClientService.deleteAllClients();

        GameClient gameClient = gameClientService.addClient("3tankista73", "fff@mail.ru", 3400);
        log.info(gameClient.toString());

        GameClient findGameClient = gameClientService.findClient(gameClient.getId());
        log.info(findGameClient.toString());

        Assertions.assertEquals(gameClient, findGameClient);
    }

	@Test
	void testNationRead() {
		nationService.deleteAllNations();

		Nation nation = nationService.addNation("СССР");
		log.info(nation.toString());

		Nation findNation = nationService.findNation(nation.getId());
		log.info(findNation.toString());

		Assertions.assertEquals(nation, findNation);
	}

	@Test
	void testClientReadNotFound() {
		gameClientService.deleteAllClients();
		Assertions.assertThrows(ClientNotFoundException.class, () -> gameClientService.findClient(-1L));
	}

	@Test
	void testLevelRead() {
		tankService.deleteAllTanks();
		tankLevelService.deleteAllLevels();

		TankLevel tankLevel = tankLevelService.addLevel(8);
		log.info(tankLevel.toString());

		TankLevel secondTankLevel = tankLevelService.addLevel(9);
		log.info(secondTankLevel.toString());

		Assertions.assertEquals(tankLevelService.findAllLevels().size(), 2);
	}

	@Test
	void testLevelReadAllEmpty() {
		tankService.deleteAllTanks();
		tankLevelService.deleteAllLevels();

		List<TankLevel> tankLevels = tankLevelService.findAllLevels();
		log.info(tankLevels.toString());

		Assertions.assertEquals(tankLevels.size(), 0);
	}

	@Test
	void testTankReadAll() {
		tankService.deleteAllTanks();
        tankLevelService.deleteAllLevels();

		TankLevel firstTankLevel = tankLevelService.addLevel(8);
		TankLevel secondTankLevel = tankLevelService.addLevel(9);

		tankService.addTank("ИС-3", nationService.addNation("СССР"), firstTankLevel, 3700000);
		tankService.addTank("E-75", nationService.addNation("Германия"), secondTankLevel, 5600000);

		List<Tank> tanks = tankService.findAllTanks();
		log.info(tanks.toString());

		Assertions.assertEquals(tanks.size(), 2);
	}

	@Test
	void testClientCreate() {
		gameClientService.deleteAllClients();

		GameClient firstGameClient = gameClientService.addClient("Barbarian", "dsfg@gmail.com", 56000);
		log.info(firstGameClient.toString());

		GameClient secondGameClient = gameClientService.addClient("KorbenDallas", "tankoviyGeniy@mail.ru", 37000);
		log.info(secondGameClient.toString());

		Assertions.assertEquals(gameClientService.findAllClients().size(), 2);
	}

	@Test
	void testUpdateTank(){
        tankService.deleteAllTanks();
        tankLevelService.deleteAllLevels();
		gameClientService.deleteAllClients();

		GameClient firstGameClient = gameClientService.addClient("Barbarian", "dsfg@gmail.com", 56000);
		GameClient secondGameClient = gameClientService.addClient("KorbenDallas", "tankoviyGeniy@mail.ru", 37000);

		TankLevel firstTankLevel = tankLevelService.addLevel(8);
		TankLevel secondTankLevel = tankLevelService.addLevel(9);

		Tank firstTank =  tankService.addTank("ИС-3", nationService.addNation("СССР"), firstTankLevel, 3700000);
		Tank secondTank = tankService.addTank("E-75", nationService.addNation("Германия"), secondTankLevel, 5600000);

		Tank tank = tankService.findTank(firstTank.getId());
		log.info(tank.toString());

		List<GameClient> newGameClient = new ArrayList<>();
		Collections.addAll(newGameClient, firstGameClient, secondGameClient);

		tank.setName("ИСУ-152");
		tank.setLevel(secondTankLevel);
		tank.setCost(4100000);
		tank.setClients(newGameClient);

		log.info(tank.toString());
	}
}
